//
//  IEMCircleManager.h
//  HyphenateChat
//
//  Created by 冯钊 on 2022/6/14.
//  Copyright © 2022 easemob.com. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "EMError.h"
#import "EMCircleUser.h"
#import "EMCircleServer.h"
#import "EMCircleChannel.h"
#import "EMCircleManagerServerDelegate.h"
#import "EMCircleManagerChannelDelegate.h"
#import "EMCursorResult.h"
#import "EMCircleServerAttribute.h"
#import "EMCircleChannelAttribute.h"
#import "EMCircleCategory.h"
#import "EMCircleVoiceChannel.h"
#import "EMCircleManagerCategoryDelegate.h"
/**
 *  \~chinese
 *  社区搜索类型：
 *  - `Name`：按社区名称搜索；
 *  - `Tag`：按社区标签搜索。
 */
typedef NS_ENUM(NSUInteger, EMCircleServerSearchType) {
    EMCircleServerSearchTypeName,
    EMCircleServerSearchTypeTag,
};

NS_ASSUME_NONNULL_BEGIN

@protocol IEMCircleManager <NSObject>

@required
/**
 *  \~chinese
 *  添加社区回调代理。
 *
 *  @param delegate  要添加的社区正常的增删改查的代理。
 *  @param queue     代理执行的队列。如果为 `nil`，则在主线程执行。
 */
- (void)addServerDelegate:(id<EMCircleManagerServerDelegate>)delegate
     queue:(nullable dispatch_queue_t)queue NS_SWIFT_NAME(add(serverDelegate:queue:));
/**
 *  \~chinese
 *  移除社区回调代理。
 *
 *  @param delegate  要移除的代理。
 */
- (void)removeServerDelegate:(id<EMCircleManagerServerDelegate>)delegate NS_SWIFT_NAME(remove(serverDelegate:));

/**
 *  \~chinese
 *  添加频道回调代理。
 *
 *  @param delegate  要添加的频道正常的增删改查的代理。
 *  @param queue     代理执行的队列。如果为 `nil`，则在主线程执行。
 */
- (void)addChannelDelegate:(id<EMCircleManagerChannelDelegate>)delegate
     queue:(nullable dispatch_queue_t)queue NS_SWIFT_NAME(add(channelDelegate:queue:));

/**
 *  \~chinese
 *  移除频道回调代理。
 *
 *  @param delegate  要移除的代理。
 */
- (void)removeChannelDelegate:(id<EMCircleManagerChannelDelegate>)delegate NS_SWIFT_NAME(remove(channelDelegate:));

/**
 *  \~chinese
 *  添加频道分组回调代理。
 *
 *  @param delegate  要添加的频道分组正常的增删改查的代理。
 *  @param queue     代理执行的队列。如果为 `nil`，则在主线程执行。
 */
- (void)addCategoryDelegate:(id<EMCircleManagerCategoryDelegate>)delegate queue:(nullable dispatch_queue_t)queue  NS_SWIFT_NAME(add(categoryDelegate:queue:));

/**
 *  \~chinese
 *  移除频道分组回调代理。
 *
 *  @param delegate  要移除的代理。
 */
- (void)removeCategoryDelegate:(id<EMCircleManagerCategoryDelegate>)delegate NS_SWIFT_NAME(remove(categoryDelegate:));
#pragma mark - server
/**
 *  \~chinese
 *  创建社区。
 *
 *  @param attribute  社区详情数据。
 *  @param completion  创建社区的回调。
 */
- (void)createServer:(EMCircleServerAttribute *)attribute completion:(void(^)(EMCircleServer * _Nullable server, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  销毁社区。
 * 
 * 仅社区所有者可调用该方法。
 *
 *  @param serverId    社区 ID。
 *  @param completion  销毁社区的回调。
 */
- (void)destroyServer:(NSString *)serverId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  修改社区信息。
 * 
 *  仅社区所有者和管理员可调用该方法。
 *
 *  @param serverId    社区 ID。
 *  @param attribute   社区详情。
 *  @param completion  社区更新回调。
 */
- (void)updateServer:(NSString *)serverId attribute:(EMCircleServerAttribute *)attribute completion:(void(^)(EMCircleServer * _Nullable server, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  加入社区。
 *
 *  @param serverId    社区 ID。
 *  @param completion  加入社区的回调。
 */
- (void)joinServer:(NSString *)serverId completion:(void(^)(EMCircleServer * _Nullable server, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  退出社区。
 *  
 *  社区成员若退出社区，则会退出该社区下的所有频道。
 * 
 *  社区所有者不支持退出社区操作，只能解散社区。
 *
 *  @param serverId    社区 ID。
 *  @param completion  退出社区的回调。
 */
- (void)leaveServer:(NSString *)serverId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  将用户移出社区。
 * 
 *  社区所有者可将所有成员移出社区，社区管理员只能将普通成员移出社区。
 *
 *  @param serverId    社区 ID。
 *  @param userId      要移出的用户 ID。
 *  @param completion  移出用户的回调。
 */
- (void)removeUserFromServer:(NSString *)serverId userId:(NSString *)userId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  邀请用户加入社区。
 *
 *  @param serverId    社区 ID。
 *  @param userId      受邀人的用户 ID。
 *  @param completion  邀请用户的回调。
 */
- (void)inviteUserToServer:(NSString *)serverId userId:(NSString *)userId welcome:(nullable NSString *)welcome completion:(void(^)(EMError * _Nullable error))completion NS_SWIFT_NAME(inviteUserToServer(serverId:userId:welcome:completion:));;

/**
 *  \~chinese
 *  同意社区加入邀请。
 *
 *  @param serverId    社区 ID。
 *  @param inviter    邀请人的用户 ID。
 *  @param completion  同意邀请的回调。
 */
- (void)acceptServerInvitation:(NSString *)serverId inviter:(NSString *)invite completion:(void(^)(EMCircleServer * _Nullable server, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  拒绝社区加入邀请。
 *
 *  @param serverId    社区 ID。
 *  @param inviter    邀请人的用户 ID。
 *  @param completion  拒绝邀请的回调。
 */
- (void)declineServerInvitation:(NSString *)serverId inviter:(NSString *)invite completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  添加社区标签。
 * 
 *  仅社区所有者和管理员可以调用该方法。
 *
 *  @param serverId    社区 ID。
 *  @param tags        要添加的标签名称列表。
 *  @param completion  添加标签的回调。
 */
- (void)addTagsToServer:(NSString *)serverId tags:(NSArray<NSString *> *)tags completion:(void(^)(NSArray<EMCircleServerTag *> * _Nullable tags, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  移除社区标签。
 * 
 *  仅社区所有者和管理员可以调用该方法。
 *
 *  @param serverId     社区 ID。
 *  @param tagIds       要删除的标签 ID 列表。
 *  @param completion   删除标签的的回调。
 */
- (void)removeTagsFromServer:(NSString *)serverId tagIds:(NSArray <NSString *>*)tagIds completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  设置社区管理员。
 * 
 *  仅社区所有者可调用该方法。
 *
 *  @param serverId 社区 ID。
 *  @param userId   要设置为社区管理员的成员的用户 ID。
 *  @param completion  设置完成的回调。
 */
- (void)addModeratorToServer:(NSString *)serverId userId:(NSString *)userId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  移除社区管理员。
 * 
 *  仅社区所有者可调用该方法。
 *
 *  @param serverId 社区 ID。
 *  @param userId   要从社区管理员列表移除的用户 ID。
 *  @param completion  设置完成的回调。
 */
- (void)removeModeratorFromServer:(NSString *)serverId userId:(NSString *)userId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  查询当前用户的社区角色。
 *
 *  @param serverId  社区 ID。
 *  @param completion  查询完成的回调。
 */
- (void)fetchSelfServerRole:(NSString *)serverId completion:(void(^)(EMCircleUserRole role, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  查询用户加入的社区列表。
 * 
 *  SDK 从指定的游标位置按照用户加入社区的正序查询。
 *
 *  @param limit 每次获取的最大用户数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从最早加入社区的开始获取。
 *  @param completion  查询完成的回调。
 */
- (void)fetchJoinedServers:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<EMCircleServer *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  查询指定社区的详情。
 *
 *  @param serverId    社区 ID。
 *  @param completion  查询完成的回调。
 */
- (void)fetchServerDetail:(NSString *)serverId completion:(void(^)(EMCircleServer * _Nullable server, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  按照关键字查询公开社区。
 *
 *  @param keyword  查询关键字。目前只支持按完整的社区名称或社区标签名称查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchServersWithKeyword:(NSString *)keyword completion:(void (^)(NSArray<EMCircleServer *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  按照关键字查询公开社区。
 * 
 *  SDK 按公开社区创建时间顺序查询。
 * 
 *  @param type  查询类型。目前只支持按社区名称或社区标签名称查询，默认根据社区名称搜索。详见 {@link EMCircleServerSearchType}。
 *  - 根据社区名称搜索，支持基于通过社区全名搜索和模糊搜索分页获取数据。若使用模糊搜索，你需要传入社区名称中最左侧的单个字或词汇，或包含该字或词汇的关键字。
 *    例如社区名称为 `足球社区01`，那么传入 `足`、`足球` 都可搜索出该社区，而使用 `球` 或 `社区01` 则搜索不到该社区。
 *  - 若根据标签名称搜索，需传入完整的标签名称，不支持模糊搜索。该类型查询直接获取带有该标签的所有社区，不支持分页查询。
 *  @param keyword  查询关键字，即社区名称或社区标签名称。
 *  @param limit 每次期望获取的社区数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从最新创建的社区的开始获取。
 *  @param completion  查询完成的回调。
 */
- (void)fetchServersWithType:(EMCircleServerSearchType)type keyword:(NSString *)keyword limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void (^)(EMCursorResult<EMCircleServer *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  分页查询社区成员列表。
 * 
 *  SDK 按成员加入社区时间的正序查询。
 *
 *  @param serverId   社区 ID。
 *  @param limit      每次期望获取的社区数量。取值范围为 [1,20]。
 *  @param cursor     查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从最早加入社区的成员开始查询。
 *  @param completion 查询完成的回调。
 */
- (void)fetchServerMembers:(NSString *)serverId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<EMCircleUser *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  查询当前用户是否加入了社区。
 *
 *  @param serverId  社区 ID。
 *  @param completion  查询完成的回调。
 */
- (void)checkSelfIsInServer:(NSString *)serverId completion:(void(^)(BOOL isJoined, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  查询当前社区的标签列表。
 *
 *  @param serverId  社区 ID。
 *  @param completion  查询完成的回调。
 */
- (void)fetchServerTags:(NSString *)serverId completion:(void(^)(NSArray<EMCircleServerTag *> * _Nullable result, EMError * _Nullable error))completion;

#pragma mark - channel
/**
 *  \~chinese
 *  在社区内创建频道。
 * 
 *  仅社区所有者可以调用该方法。
 *
 *  @param serverId   社区 ID。
 *  @param attribute  频道属性配置。
 *  @param categoryId 频道分组 ID。若传入 `nil`，该频道归为默认频道分组。
 *  @param mode   频道模式，即文字频道或语聊频道。
 *  @param completion  创建频道的回调。
 */
- (void)createChannel:(NSString *)serverId categoryId:(nullable NSString *)categoryId attribute:(EMCircleChannelAttribute *)attribute mode:(EMCircleChannelMode)mode completion:(void(^)(EMCircleChannel * _Nullable channel, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  销毁社区内的频道。
 * 
 *  仅社区所有者可调用该方法。
 *
 *  @param serverId   社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion 销毁频道的回调。
 */
- (void)destroyChannel:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  修改频道信息。
 * 
 *  仅社区所有者和管理员可调用该方法。
 *
 *  @param serverId    社区 ID。
 *  @param channelId   频道 ID。
 *  @param attribute   频道详情。
 *  @param completion  修改完成的回调。
 */
- (void)updateChannel:(NSString *)serverId channelId:(NSString *)channelId attribute:(EMCircleChannelAttribute *)attribute completion:(void(^)(EMCircleChannel * _Nullable channel, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  加入频道。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion  加入频道的回调。
 */
- (void)joinChannel:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(EMCircleChannel * _Nullable channel, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  退出频道。
 * 
 *  退出频道的成员不会再收到频道消息。
 *
 *  社区内的默认频道不允许成员主动退出。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion  退出频道的回调。
 */
- (void)leaveChannel:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 * 移除频道成员。
 * 
 * 社区所有者可将频道中的所有成员移出，社区管理员只能将普通成员移出频道。
 * 
 * 社区内的默认频道不允许踢出成员。
 *
 *  @param serverId   社区 ID。
 *  @param channelId  频道 ID。
 *  @param userId     要移除的成员的用户 ID。
 *  @param completion 移除用户的回调。
 */
- (void)removeUserFromChannel:(NSString *)serverId channelId:(NSString *)channelId userId:(NSString *)userId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  邀请用户加入社区中的频道。
 *
 *  @param serverId    社区 ID。
 *  @param channelId   频道 ID。
 *  @param userId      受邀用户的用户 ID。
 *  @param completion  邀请用户的回调。
 */
- (void)inviteUserToChannel:(NSString *)serverId channelId:(NSString *)channelId userId:(NSString *)userId welcome:(nullable NSString *)welcome completion:(void(^)(EMError * _Nullable error))completion NS_SWIFT_NAME(inviteUserToChannel(serverId:channelId:userId:welcome:completion:));

/**
 *  \~chinese
 *  同意加入频道的邀请。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param inviter  邀请人的用户 ID。
 *  @param completion  同意邀请的回调。
 */
- (void)acceptChannelInvitation:(NSString *)serverId channelId:(NSString *)channelId inviter:(NSString *)inviter completion:(void(^)(EMCircleChannel * _Nullable channel, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  拒绝加入频道的邀请。
 *
 *  @param serverId   社区 ID。
 *  @param channelId  频道 ID。
 *  @param inviter    邀请人的用户 ID。
 *  @param completion 拒绝邀请的回调。
 */
- (void)declineChannelInvitation:(NSString *)serverId channelId:(NSString *)channelId inviter:(NSString *)inviter completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  禁言频道中的成员。
 * 
 *  仅社区所有者和社区管理员可以调用该方法。
 *
 *  @param userId  要禁言的成员的用户 ID。
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param duration 禁言时长，单位为毫秒。值为 `-1` 时表示永久禁言。
 *  @param completion  禁言回调。
 */
- (void)muteUserInChannel:(NSString *)userId serverId:(NSString *)serverId channelId:(NSString *)channelId duration:(NSUInteger)duration completion:(void(^)(EMError * _Nullable error))completion NS_SWIFT_NAME(muteUserInChannel(userId:serverId:channelId:duration:completion:));

/**
 *  \~chinese
 *  解除禁言频道中的用户。
 * 
 *  仅社区所有者和社区管理员可以调用该方法。
 *
 *  @param userId  要解除禁言的用户 ID。
 *  @param serverId   社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion  解除禁言的回调。
 */
- (void)unmuteUserInChannel:(NSString *)userId serverId:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(EMError * _Nullable error))completion NS_SWIFT_NAME(unmuteUserInChannel(userId:serverId:channelId:completion:));

/**
 *  \~chinese
 *  获取指定频道的详情。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion  查询完成的回调。
 */
- (void)fetchChannelDetail:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(EMCircleChannel * _Nullable channel, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  分页获取指定社区中的公开频道列表。
 * 
 *  SDK 按公开频道的创建时间的正序获取。
 *
 *  @param serverId  社区 ID。
 *  @param limit     每次获取的最大公开频道数量。取值范围为 [1,20]。
 *  @param cursor    查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从最早创建的公开频道开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchPublicChannelsInServer:(NSString *)serverId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<EMCircleChannel *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  分页获取指定社区中私密频道列表。
 * 
 *  SDK 按照私密频道创建时间的正序返回。
 *
 *  @param serverId  社区 ID。
 *  @param limit     每次期望获取的私密频道数量。取值范围为 [1,20]。
 *  @param cursor    查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从创建最早的私密频道开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchPrivateChannelsInServer:(NSString *)serverId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<EMCircleChannel *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  分页获取指定频道分组中的公开频道列表。
 * 
 *  SDK 按照公开频道创建时间的正序返回。
 *
 *  @param serverId  社区 ID。
 *  @param categoryId  频道分组 ID。
 *  @param limit  每次期望获取的公开频道数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从最早创建的公开频道开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchPublicChannelsInCategory:(NSString *)serverId categoryId:(NSString *)categoryId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void (^)(EMCursorResult<EMCircleChannel *> * _Nullable, EMError * _Nullable))completion;

/**
 *  \~chinese
 *  分页获取频道分组中的私密频道列表。
 * 
 *  SDK 按照私密频道创建时间的正序返回。
 *
 *  @param serverId  社区 ID。
 *  @param categoryId  频道分组 ID。
 *  @param limit  每次期望获取的私密频道数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从创建最早的私密频道开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchPrivateChannelsInCategory:(NSString *)serverId categoryId:(NSString *)categoryId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void (^)(EMCursorResult<EMCircleChannel *> * _Nullable, EMError * _Nullable))completion;

/**
 *  \~chinese
 *  分页获取频道中的成员列表。
 * 
 *  SDK 从指定位置按照成员加入频道的时间正序获取。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param limit 每次期望获取的成员数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从最早加入频道的成员开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchChannelMembers:(NSString *)serverId channelId:(NSString *)channelId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<EMCircleUser *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  分页获取社区中已加入的频道。
 * 
 *  SDK 从指定位置按照频道创建时间的正序获取。
 *
 *  @param serverId  社区 ID。
 *  @param limit  每次期望获取的频道数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。
 *                 首次查询传入 `nil` 或 @""，SDK 从创建最早的加入频道开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchJoinedChannelIdsInServer:(NSString *)serverId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<NSString *> * _Nullable result, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  查询当前用户是否加入了频道。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion  查询完成的回调。
 */
- (void)checkSelfIsInChannel:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(BOOL isJoined, EMError * _Nullable error))completion NS_SWIFT_NAME(checkSelfIsInChannel(serverId:channelId:completion:));

/**
 *  \~chinese
 *  查询频道的禁言列表。
 *
 *  @param serverId  社区 ID。
 *  @param channelId  频道 ID。
 *  @param completion  查询完成的回调。
 */
- (void)fetchChannelMuteUsers:(NSString *)serverId channelId:(NSString *)channelId completion:(void(^)(NSDictionary<NSString *, NSNumber *> * _Nullable muteInfo, EMError * _Nullable error))completion;

#pragma mark - category
/**
 *  \~chinese
 *  创建频道分组。
 * 
 *  仅社区所有者可以调用该方法。
 *
 *  @param serverId  社区 ID。
 *  @param name  频道分组名称。
 *  @param completion  创建频道分组的回调。
 */
- (void)createCategory:(NSString *)serverId name:(NSString *)name completion:(void(^)(EMCircleCategory * _Nullable category, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  删除频道分组。
 * 
 *  仅社区所有者和管理员可调用该方法。
 * 
 *  删除频道分组后，该频道分组下的所有频道会添加到社区的默认频道分组中。
 *
 *  @param serverId    社区 ID。
 *  @param categoryId    频道分组 ID。
 *  @param completion  销毁频道分组的回调。
 */
- (void)destroyCategory:(NSString *)serverId categoryId:(NSString *)categoryId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  修改频道分组名称。
 * 
 *  仅社区所有者和管理员可调用该方法。
 *
 *  @param serverId    社区 ID。
 *  @param categoryId  频道分组 ID。
 *  @param name        修改后的频道分组名称。
 *  @param completion  频道分组更新回调。
 */
- (void)updateCategory:(NSString *)serverId categoryId:(NSString *)categoryId name:(NSString *)name completion:(void(^)(EMCircleCategory * _Nullable category, EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  更换指定频道所属的频道分组。
 * 
 *  仅社区所有者和管理员可以调用该方法。
 *
 *  @param serverId    社区 ID。
 *  @param channelId    频道 ID。
 *  @param newCategoryId   新的频道分组 ID。传 `nil` 或空字符串 @"" 时，表示将频道转移至社区下的默认频道分组 ID。
 *  @param completion  更换频道分组回调。
 */
- (void)transferChannel:(NSString *)serverId channelId:(NSString *)channelId newCategoryId:(nullable NSString *)newCategoryId completion:(void(^)(EMError * _Nullable error))completion;

/**
 *  \~chinese
 *  获取社区下的频道分组列表。
 * 
 * SDK 按频道分组的创建时间正序查询。
 *
 *  @param serverId    社区 ID。
 *  @param limit  每次期望获取的频道分组数量。取值范围为 [1,20]。
 *  @param cursor  查询的游标位置，从上次查询的结果中获取。首次查询传入 `nil` 或 @""，SDK 从创建最早的频道分组开始查询。
 *  @param completion  查询完成的回调。
 */
- (void)fetchCategoriesInServer:(NSString *)serverId limit:(NSUInteger)limit cursor:(nullable NSString *)cursor completion:(void(^)(EMCursorResult<EMCircleCategory *> * _Nullable result, EMError * _Nullable error))completion;

@end

NS_ASSUME_NONNULL_END
